import csv, matplotlib.pyplot as plt, numpy, copy, scipy, scipy.stats
import os, os.path
from datetime import datetime

numpy.set_printoptions(precision=3,linewidth=300,suppress =True,threshold=10000)

def sign(x):
	if x>0:
		return 1
	elif x<0:
		return -1
	else:
		return 0
def avgerror(x):
	return float(numpy.sum(x==-1))/numpy.sum(x!=0)

def avgerasure(x):
	return float(numpy.sum(x==0))/numpy.sum(x < numpy.infty)

def neterror(x):
	total_right = numpy.sum(x==1)
	total_wrong = numpy.sum(x==-1)
	total_erasures = numpy.sum(x==0)
	avg = (0*total_right + .5*total_erasures + 1*total_wrong)/(total_right + total_erasures + total_wrong)
	std = numpy.sqrt( (total_right * (0-avg)**2 + total_erasures * (.5-avg)**2 + total_wrong * (1-avg)**2) /(total_right + total_erasures + total_wrong))
	return avg, std

def count_inconsistency(x, e):
	total_consistent = 0
	total_inconsistent = 0
	total_erasures = 0
	for row in range(len(x[:,0])):
		for col in range(len(x[0,:])):
			if row < col:
				total_consistent += x[row,col]*(x[row,col]-1)*.5 + x[col,row]*(x[col,row]-1)*.5
				total_inconsistent += x[col,row]*x[row,col]
				total_erasures += (x[col,row]+x[row,col])*e[row,col] + e[col,row]*(e[col,row]-1)*.5
	avg = (0*total_consistent + .5*total_erasures + 1*total_inconsistent)/(total_consistent + total_erasures + total_inconsistent)
	std = numpy.sqrt( (total_consistent * (0-avg)**2 + total_erasures * (.5-avg)**2 + total_inconsistent * (1-avg)**2) /(total_consistent + total_erasures + total_inconsistent))
	return avg, std

class experiment_result:
	def __init__(self, csv_filename):
		self.num_questions=0 #cardinal number of questions

		self.csv_filename=csv_filename
		self.num_workers_cardinal=0 #number of workers in the cardinal case
		self.num_workers_ordinal=0 #number of workers in the ordinal case
		self.num_questions_cardinal = 0
		self.num_questions_ordinal = 0
		self.answers_ordinal=[]
		self.answers_cardinal=[]
		
		csvfile = open(csv_filename+'.csv','rU')
		reader = list(csv.reader(csvfile,delimiter=','))#,quotechar='')
		header = reader[0]
		reader.pop(0)		
		reader = list(reader)
				
		#GET DATA FROM HEADER
		col_question_type = -1
		cols_trueAnsL_ord = [] #set of columns containing identity of left item in each ordinal question
		cols_trueAnsR_ord = [] #set of columns containing identity of right item in each ordinal question
		cols_trueAns_card = [] #set of columns containing identity of each item in each cardinal question
		cols_answer_ord = [] #set of columns containing answers given by worker to ordinal questions
		cols_answer_card = [] #set of columns containing all answers given by worker to cardinal questions
		col_work_time = -1

		#identify which columns in csv contain what
		for colnum in range(len(header)):
			col = header[colnum]
			if col.startswith('Answer.trueAnsL') or col.startswith('Answer.questionIndexL'):
				cols_trueAnsL_ord = numpy.int_(numpy.append(cols_trueAnsL_ord, colnum))
			elif col.startswith('Answer.trueAnsR') or col.startswith('Answer.questionIndexR'):
				cols_trueAnsR_ord = numpy.int_(numpy.append(cols_trueAnsR_ord, colnum))
			elif col.startswith('Answer.trueAns') or col.startswith('Answer.questionIndex'):
				cols_trueAns_card = numpy.int_(numpy.append(cols_trueAns_card, colnum))
			elif col.startswith('Answer.question_type'):
				col_question_type = colnum
			elif col.startswith('Answer.work_time'):
				col_work_time = colnum
	
		self.task_type = 'objective'
		if header[cols_trueAnsL_ord[0]].startswith('Answer.questionIndexL'):
			self.task_type = 'subjective'

		self.num_questions_ordinal = len(cols_trueAnsL_ord)
		self.num_questions_cardinal = len(cols_trueAns_card)
   
		for colnum in range(len(header)):
			col = header[colnum]
			if col.startswith('Answer.answer'):
				cols_answer_card = numpy.int_(numpy.append(cols_answer_card, colnum))
				if numpy.int(col[13:]) < self.num_questions_ordinal:
						cols_answer_ord = numpy.int_(numpy.append(cols_answer_ord, colnum))

		#count number of cardinal/ordinal workers
		for row in reader:
				if row[col_question_type] == 'ordinal':
					self.num_workers_ordinal += 1
				else:
					self.num_workers_cardinal += 1

		#Now extract actual data
		self.answers_ordinal = numpy.zeros((self.num_workers_ordinal,self.num_questions_ordinal))#all ordinal answers given by all workers
		self.answers_cardinal = numpy.zeros((self.num_workers_cardinal,self.num_questions_cardinal))#all cardinal answers given by all workers
		self.trueLeft_ordinal = numpy.zeros((self.num_workers_ordinal, self.num_questions_ordinal))#true ordinal answers left
		self.trueRight_ordinal = numpy.zeros((self.num_workers_ordinal, self.num_questions_ordinal))#true ordinal answers right
		self.true_cardinal = numpy.zeros((self.num_workers_cardinal, self.num_questions_cardinal))#true cardinal answers

		#for total time between worker accepting job and worker submitting answer
		self.time_ordinal = numpy.array([])
		self.time_cardinal = numpy.array([])

		cur_ordinal = int(0)
		cur_cardinal = int(0)
		
		for row in reader:
			if row[col_question_type] == 'cardinal':
				for col in range(self.num_questions_cardinal):
					if row[cols_answer_card[col]]=='':#if some answer is invalid then make this worker invalid
						self.num_workers_cardinal-=1
						self.answers_cardinal = numpy.delete(self.answers_cardinal,self.num_cardinal,0)		
						self.true_cardinal = numpy.delete(self.true_cardinal,self.num_cardinal,0)
						cur_cardinal -= 1
						break
					self.answers_cardinal[cur_cardinal, col] = row[cols_answer_card[col]]
					self.true_cardinal[cur_cardinal, col] = numpy.float(row[cols_trueAns_card[col]])
					self.time_cardinal = numpy.append(self.time_cardinal, numpy.int(row[col_work_time]))
				cur_cardinal += 1
			#	
			elif row[col_question_type] == 'ordinal':
				for col in range(self.num_questions_ordinal):
					if row[cols_answer_ord[col]]=='':#if some answer is invalid then make this worker invalid
						self.num_workers_ordinal-=1
						self.answers_ordinal = numpy.delete(self.answers_ordinal,self.num_ordinal,0)		
						self.trueLeft_ordinal = numpy.delete(self.trueLeft_ordinal,self.num_ordinal,0)
						self.trueRight_ordinal = numpy.delete(self.trueRight_ordinal,self.num_ordinal,0)
						cur_ordinal -= 1
						break
					elif row[cols_answer_ord[col]] == 'L':
						row[cols_answer_ord[col]]=-1
					elif row[cols_answer_ord[col]] == 'R':
						row[cols_answer_ord[col]]=+1
					self.answers_ordinal[cur_ordinal, col] = row[cols_answer_ord[col]]
					self.trueLeft_ordinal[cur_ordinal, col] = numpy.float(row[cols_trueAnsL_ord[col]])
					self.trueRight_ordinal[cur_ordinal, col] = numpy.float(row[cols_trueAnsR_ord[col]])
					self.time_ordinal = numpy.append(self.time_ordinal, numpy.int(row[col_work_time]))
				cur_ordinal += 1
    
		#Compute Inconsistencies
		self.errors_ordinal = numpy.zeros((self.num_workers_ordinal,self.num_questions_ordinal))
		self.errors_cardinal = numpy.zeros((self.num_workers_cardinal,self.num_questions_cardinal/2))

		self.consistency_ordinal=numpy.zeros((self.num_questions_cardinal,self.num_questions_cardinal)) #element (i,j) is number of times i has beaten j
		self.consistency_cardinal=numpy.zeros((self.num_questions_cardinal,self.num_questions_cardinal)) #element (i,j) is number of times i has beaten j
		self.consistency_ordinal_erasures=numpy.zeros((self.num_questions_cardinal,self.num_questions_cardinal)) #element (i,j) is number of times i has beaten j
		self.consistency_cardinal_erasures=numpy.zeros((self.num_questions_cardinal,self.num_questions_cardinal)) #element (i,j) is number of times i has beaten j

		for row in range(self.num_workers_ordinal):
			for col in range(self.num_questions_ordinal):
				if self.task_type == 'objective':
					self.errors_ordinal[row,col]=sign((self.trueRight_ordinal[row,col]-self.trueLeft_ordinal[row,col])*self.answers_ordinal[row,col])
				elif self.task_type == 'subjective':
					if self.answers_ordinal[row,col] < 0:
							self.consistency_ordinal[int(self.trueLeft_ordinal[row,col]),int(self.trueRight_ordinal[row,col])]+=1
					elif self.answers_ordinal[row,col] > 0:
							self.consistency_ordinal[int(self.trueRight_ordinal[row,col]),int(self.trueLeft_ordinal[row,col])]+=1					
				
		for row in range(self.num_workers_cardinal):
			for col in range(self.num_questions_cardinal/2):
				if self.task_type == 'objective':
					self.errors_cardinal[row,col]=sign((self.true_cardinal[row,2*col]-self.true_cardinal[row,(2*col+1)%self.num_questions_cardinal])*(self.answers_cardinal[row,2*col]-self.answers_cardinal[row,((2*col+1))%self.num_questions_cardinal]))
				elif self.task_type == 'subjective':
					if self.answers_cardinal[row,2*col] > self.answers_cardinal[row,2*col+1]:
						self.consistency_cardinal[self.true_cardinal[row,2*col],self.true_cardinal[row,2*col+1]]+=1
					elif self.answers_cardinal[row,2*col+1] > self.answers_cardinal[row,2*col]:
						self.consistency_cardinal[self.true_cardinal[row,2*col+1],self.true_cardinal[row,2*col]]+=1
					else:
						self.consistency_cardinal_erasures[self.true_cardinal[row,2*col+1],self.true_cardinal[row,2*col]]+=0.5
						self.consistency_cardinal_erasures[self.true_cardinal[row,2*col],self.true_cardinal[row,2*col+1]]+=0.5

EXPERIMENT_vals = ['taglines','circle_area', 'spelling_mistakes','photo_age','city_distances','piano',  'search_relevance']

for EXPERIMENT in EXPERIMENT_vals:
	data = experiment_result(EXPERIMENT + '/' + EXPERIMENT + '_data')
	print '%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% \nEXPERIMENT = '+ EXPERIMENT
			
	print 'time for ordinal (seconds)',numpy.median(data.time_ordinal), float(numpy.std(data.time_ordinal))/numpy.sqrt(data.num_workers_ordinal)
	print 'time for cardinal (seconds)',numpy.median(data.time_cardinal), float(numpy.std(data.time_cardinal))/numpy.sqrt(data.num_workers_cardinal)
	
	